"""empty message

Revision ID: c81f3e83bdb5
Revises: 43d0cf0ed5e7
Create Date: 2019-05-15 23:15:01.050692

"""

from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "c81f3e83bdb5"
down_revision = "43d0cf0ed5e7"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column(
        "organisation",
        sa.Column("use_original_file_name", sa.Boolean(), nullable=True),
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("organisation", "use_original_file_name")
    # ### end Alembic commands ###
