"""empty message

Revision ID: 38baa9a23b3d
Revises: 443d1e78a932
Create Date: 2018-08-14 15:48:04.225525

"""

from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = "38baa9a23b3d"
down_revision = "443d1e78a932"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column("task", sa.Column("priority", sa.Integer(), nullable=True))
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("task", "priority")
    # ### end Alembic commands ###
